@@ -5,7 +5,6 @@
  */
 
 #include <gtk/gtk.h>
-#include "gtk/gtkrendernodepaintableprivate.h"
 
 static guint tick_cb;
 static GtkAdjustment *hadjustment;
@@ -44,7 +43,7 @@ scroll_cb (GtkWidget *widget,
   return G_SOURCE_CONTINUE;
 }
 
-extern GtkWidget *create_icon (void);
+extern GtkWidget *create_icon_by_id (gsize id);
 
 static void
 populate_icons (void)
@@ -63,7 +62,7 @@ populate_icons (void)
 
   for (top = 0; top < 100; top++)
     for (left = 0; left < 15; left++)
-      gtk_grid_attach (GTK_GRID (grid), create_icon (), left, top, 1, 1);
+      gtk_grid_attach (GTK_GRID (grid), create_icon_by_id (top * 15 + left), left, top, 1, 1);
 
   hincrement = 0;
   vincrement = 5;
@@ -454,7 +453,7 @@ populate_symbolics (void)
 }
 
 GtkWidget *
-create_path (void)
+create_svg (void)
 {
   GtkWidget *image;
   static int idx = 0;
@@ -477,7 +476,7 @@ create_path (void)
 }
 
 static void
-populate_paths (void)
+populate_svg (void)
 {
   GtkWidget *grid;
   int top, left;
@@ -494,7 +493,7 @@ populate_paths (void)
   for (top = 0; top < 100; top++)
     for (left = 0; left < 15; left++)
        {
-         gtk_grid_attach (GTK_GRID (grid), create_path (), left, top, 1, 1);
+         gtk_grid_attach (GTK_GRID (grid), create_svg (), left, top, 1, 1);
        }
 
   hincrement = 0;
@@ -510,8 +509,7 @@ static GtkWidget *
 create_squiggle (void)
 {
   GtkWidget *image;
-  graphene_rect_t bounds = GRAPHENE_RECT_INIT (0, 0, 18, 18);
-  GskRenderNode *child, *node;
+  GtkSnapshot *snapshot;
   GdkPaintable *paintable;
   GskStroke *stroke;
   GskPathBuilder *builder;
@@ -534,9 +532,9 @@ create_squiggle (void)
   path = gsk_path_builder_free_to_path (builder);
   stroke = gsk_stroke_new (1);
 
-  child = gsk_color_node_new (&(GdkRGBA) { 0, 0, 0, 1}, &bounds);
-  node = gsk_stroke_node_new (child, path, stroke);
-  paintable = gtk_render_node_paintable_new (node, &bounds);
+  snapshot = gtk_snapshot_new ();
+  gtk_snapshot_append_stroke (snapshot, path, stroke, &(GdkRGBA) { 0, 0, 0, 1});
+  paintable = gtk_snapshot_free_to_paintable (snapshot, &GRAPHENE_SIZE_INIT (18, 18));
 
   image = gtk_image_new ();
   gtk_image_set_icon_size (GTK_IMAGE (image), GTK_ICON_SIZE_LARGE);
@@ -545,8 +543,6 @@ create_squiggle (void)
   g_object_unref (paintable);
   gsk_stroke_free (stroke);
   gsk_path_unref (path);
-  gsk_render_node_unref (node);
-  gsk_render_node_unref (child);
 
   return image;
 }
@@ -645,8 +641,8 @@ set_widget_type (int type)
       break;
 
     case 10:
-      gtk_window_set_title (GTK_WINDOW (window), "Scrolling paths");
-      populate_paths ();
+      gtk_window_set_title (GTK_WINDOW (window), "Scrolling SVG");
+      populate_svg ();
       break;
 
     case 11:
