@@ -127,7 +127,7 @@
 #define SINGLE_PIXEL_BUFFER_VERSION     1
 #define SYSTEM_BELL_VERSION             1
 #define CURSOR_SHAPE_VERSION            1
-#define GTK_SHELL1_VERSION              6
+#define GTK_SHELL1_VERSION              7
 #define XDG_WM_DIALOG_VERSION           1
 #define XDG_TOPLEVEL_ICON_VERSION       1
 #define XDG_WM_BASE_VERSION             7
@@ -889,11 +889,12 @@ gdk_wayland_display_dispose (GObject *object)
 {
   GdkWaylandDisplay *display_wayland = GDK_WAYLAND_DISPLAY (object);
 
-  g_list_free_full (display_wayland->toplevels, destroy_toplevel);
+  g_clear_list (&display_wayland->toplevels, destroy_toplevel);
 
   gdk_wayland_display_uninstall_gsources (display_wayland);
 
-  g_list_free_full (display_wayland->async_roundtrips, (GDestroyNotify) wl_callback_destroy);
+  g_clear_list (&display_wayland->async_roundtrips,
+                (GDestroyNotify) wl_callback_destroy);
 
   if (display_wayland->known_globals)
     {
@@ -901,7 +902,7 @@ gdk_wayland_display_dispose (GObject *object)
       display_wayland->known_globals = NULL;
     }
 
-  g_list_free_full (display_wayland->on_has_globals_closures, g_free);
+  g_clear_list (&display_wayland->on_has_globals_closures, g_free);
 
   g_clear_pointer (&display_wayland->compositor, wl_compositor_destroy);
   g_clear_pointer (&display_wayland->xdg_wm_base, xdg_wm_base_destroy);
