@@ -205,10 +205,9 @@ gsk_fill_node_new (GskRenderNode *child,
   self->path = gsk_path_ref (path);
   self->fill_rule = fill_rule;
 
-  if (gsk_path_get_bounds (path, &path_bounds))
-    gsk_rect_intersection (&path_bounds, &child->bounds, &node->bounds);
-  else
-    gsk_rect_init (&node->bounds, 0.f, 0.f, 0.f, 0.f);
+  if (!gsk_path_get_bounds (path, &path_bounds) || 
+      !gsk_rect_intersection (&path_bounds, &child->bounds, &node->bounds))
+    node->bounds = GRAPHENE_RECT_INIT (0, 0, 0, 0);
 
   return node;
 }
