@@ -20,6 +20,8 @@
 #include "gdktextureutilsprivate.h"
 #include "gtkscalerprivate.h"
 #include "gtksnapshot.h"
+#include "gtk/gtksvg.h"
+#include "gtk/gtksymbolicpaintable.h"
 
 #include "gdk/gdktextureprivate.h"
 #include "gdk/loaders/gdkpngprivate.h"
@@ -28,102 +30,43 @@
 #include "gtk/gtkenums.h"
 #include "gtk/gtkbitmaskprivate.h"
 
-#include <librsvg/rsvg.h>
-
 /* {{{ svg helpers */
 
-static gboolean
-gdk_texture_get_rsvg_handle_size (RsvgHandle *handle, gdouble *out_width, gdouble *out_height)
-{
-#if LIBRSVG_CHECK_VERSION (2,52,0)
-  gboolean has_viewbox;
-  RsvgRectangle viewbox;
-
-  if (rsvg_handle_get_intrinsic_size_in_pixels (handle, out_width, out_height))
-    return TRUE;
-
-  rsvg_handle_get_intrinsic_dimensions (handle,
-                                        NULL, NULL, NULL, NULL,
-                                        &has_viewbox,
-                                        &viewbox);
-
-  if (has_viewbox)
-    {
-      *out_width = viewbox.width;
-      *out_height = viewbox.height;
-      return TRUE;
-    }
-
-  return FALSE;
-#else
-  RsvgDimensionData dim;
-  rsvg_handle_get_dimensions (handle, &dim);
-  if (out_width)
-    *out_width = dim.width;
-  if (out_height)
-    *out_height = dim.height;
-  return TRUE;
-#endif
-}
-
 static GdkTexture *
-gdk_texture_new_from_rsvg (RsvgHandle  *handle,
-                           int          width,
-                           int          height,
-                           GError     **error)
+svg_to_texture (GtkSvg  *svg,
+                int      width,
+                int      height,
+                GdkRGBA *colors,
+                size_t   n_colors)
 {
-  int stride;
-  guchar *data;
+  GtkSnapshot *snapshot;
+  GskRenderNode *node;
   cairo_surface_t *surface;
   cairo_t *cr;
-  GdkTexture *texture = NULL;
+  GdkTexture *texture;
 
-  stride = width * 4;
-  data = g_new0 (guchar, stride * height);
+  surface = cairo_image_surface_create (CAIRO_FORMAT_ARGB32, width, height);
 
-  surface = cairo_image_surface_create_for_data (data,
-                                                 CAIRO_FORMAT_ARGB32,
-                                                 width, height,
-                                                 stride);
+  snapshot = gtk_snapshot_new ();
 
-  cr = cairo_create (surface);
+  gtk_symbolic_paintable_snapshot_symbolic (GTK_SYMBOLIC_PAINTABLE (svg),
+                                            snapshot,
+                                            width, height,
+                                            colors, n_colors);
 
-#if !LIBRSVG_CHECK_VERSION (2,52,0)
-  {
-    RsvgDimensionData dim;
-    gdouble sx,sy,s;
+  node = gtk_snapshot_free_to_node (snapshot);
 
-    rsvg_handle_get_dimensions (handle, &dim);
-    sx = (gdouble)width / dim.width;
-    sy = (gdouble)height / dim.height;
-    s = MIN (sx, sy);
-
-    cairo_scale (cr, s, s);
-  }
-
-  if (!rsvg_handle_render_cairo (handle, cr))
-    g_set_error (error, GDK_TEXTURE_ERROR, GDK_TEXTURE_ERROR_CORRUPT_IMAGE,
-                 "Error rendering SVG document (%s)", cairo_status_to_string (cairo_status (cr)));
-  else
-#else
-  if (rsvg_handle_render_document (handle, cr,
-                                   &(RsvgRectangle) { 0, 0, width, height },
-                                   error))
-#endif
+  if (node)
     {
-      GBytes *bytes;
+      cr = cairo_create (surface);
+      gsk_render_node_draw (node, cr);
+      cairo_destroy (cr);
 
-      bytes = g_bytes_new (data, stride * height);
-      texture = gdk_memory_texture_new (width, height,
-                                        GDK_MEMORY_DEFAULT,
-                                        bytes,
-                                        stride);
-      g_bytes_unref (bytes);
+      gsk_render_node_unref (node);
     }
 
-  cairo_destroy (cr);
+  texture = gdk_texture_new_for_surface (surface);
   cairo_surface_destroy (surface);
-  g_free (data);
 
   return texture;
 }
@@ -131,33 +74,32 @@ gdk_texture_new_from_rsvg (RsvgHandle  *handle,
 static GdkTexture *
 gdk_texture_new_from_svg_bytes (GBytes  *bytes,
                                 double   scale,
+                                GdkRGBA *colors,
+                                size_t   n_colors,
                                 GError **error)
 {
-  const guchar *data;
-  gsize len;
-  RsvgHandle *handle;
+  GtkSvg *svg;
   GdkTexture *texture;
   int width, height;
-  double w, h;
 
-  data = g_bytes_get_data (bytes, &len);
-  handle = rsvg_handle_new_from_data (data, len, error);
-  if (!handle)
-    return NULL;
+  svg = gtk_svg_new_from_bytes (bytes);
 
-  if (!gdk_texture_get_rsvg_handle_size (handle, &w, &h))
+  width = gdk_paintable_get_intrinsic_width (GDK_PAINTABLE (svg));
+  height = gdk_paintable_get_intrinsic_height (GDK_PAINTABLE (svg));
+
+  if (width == 0 || height == 0)
     {
       g_set_error_literal (error, G_IO_ERROR, G_IO_ERROR_FAILED,
                            "Svg image has no intrinsic size; please set one");
-      g_object_unref (handle);
+      g_object_unref (svg);
       return NULL;
     }
 
-  width = ceil (w * scale);
-  height = ceil (h * scale);
-
-  texture = gdk_texture_new_from_rsvg (handle, width, height, error);
-  g_object_unref (handle);
+  texture = svg_to_texture (svg,
+                            ceil (scale * width),
+                            ceil (scale * height),
+                            colors, n_colors);
+  g_object_unref (svg);
 
   return texture;
 }
@@ -165,44 +107,8 @@ gdk_texture_new_from_svg_bytes (GBytes  *bytes,
 /* }}} */
 /* {{{ Symbolic processing */
 
-static char *
-make_stylesheet (const char *fg_color,
-                 const char *success_color,
-                 const char *warning_color,
-                 const char *error_color)
-{
-  return g_strconcat ("rect,circle,path,.foreground-fill {\n"
-                      " fill:", fg_color, "!important;\n"
-                      "}\n"
-                      ".warning,.warning-fill {\n"
-                      " fill:", warning_color, "!important;\n"
-                      "}\n"
-                      ".error,.error-fill {\n"
-                      "  fill:", error_color, "!important;\n"
-                      "}\n"
-                      ".success,.success-fill {\n"
-                      "  fill:", success_color, "!important;\n"
-                      "}\n"
-                      ".transparent-fill {\n"
-                      "  fill: none !important;\n"
-                      "}\n"
-                      ".foreground-stroke {\n"
-                      "  stroke:", fg_color, "!important;\n"
-                      "}\n"
-                      ".warning-stroke {\n"
-                      "  stroke:", warning_color, "!important;\n"
-                      "}\n"
-                      ".error-stroke {\n"
-                      "  stroke:", error_color, "!important;\n"
-                      "}\n"
-                      ".success-stroke {\n"
-                      "  stroke:", success_color, "!important;\n"
-                      "}",
-                      NULL);
-}
-
 static GdkTexture *
-load_symbolic_svg (RsvgHandle  *handle,
+load_symbolic_svg (GtkSvg      *svg,
                    int          width,
                    int          height,
                    const char  *fg_color,
@@ -211,23 +117,14 @@ load_symbolic_svg (RsvgHandle  *handle,
                    const char  *error_color,
                    GError     **error)
 {
-  GdkTexture *texture = NULL;
-  char *stylesheet;
+  GdkRGBA colors[4];
 
-  stylesheet = make_stylesheet (fg_color, success_color, warning_color, error_color);
+  gdk_rgba_parse (&colors[GTK_SYMBOLIC_COLOR_FOREGROUND], fg_color);
+  gdk_rgba_parse (&colors[GTK_SYMBOLIC_COLOR_SUCCESS], success_color);
+  gdk_rgba_parse (&colors[GTK_SYMBOLIC_COLOR_WARNING], warning_color);
+  gdk_rgba_parse (&colors[GTK_SYMBOLIC_COLOR_ERROR], error_color);
 
-  if (!rsvg_handle_set_stylesheet (handle, (const guint8 *) stylesheet, strlen (stylesheet), error))
-    {
-      g_prefix_error (error, "Could not set stylesheet");
-      goto out;
-    }
-
-  texture = gdk_texture_new_from_rsvg (handle, width, height, error);
-
-out:
-  g_free (stylesheet);
-
-  return texture;
+  return svg_to_texture (svg, width, height, colors, 4);
 }
 
 static gboolean
@@ -344,8 +241,7 @@ gdk_texture_new_from_bytes_symbolic (GBytes    *bytes,
                                      GError   **error)
 
 {
-  RsvgHandle *handle;
-  double w, h;
+  GtkSvg *svg;
   const char *r_string = "rgb(255,0,0)";
   const char *g_string = "rgb(0,255,0)";
   gboolean only_fg;
@@ -353,28 +249,24 @@ gdk_texture_new_from_bytes_symbolic (GBytes    *bytes,
   GBytes *data_bytes;
   GdkTexture *texture;
 
-  handle = rsvg_handle_new_from_data (g_bytes_get_data (bytes, NULL),
-                                      g_bytes_get_size (bytes),
-                                      error);
+  svg = gtk_svg_new_from_bytes (bytes);
 
-  if (width == 0 || height == 0)
+  if (width == 0 && height == 0)
     {
-      /* Fetch size from the original icon */
-      if (!gdk_texture_get_rsvg_handle_size (handle, &w, &h))
+      width = gdk_paintable_get_intrinsic_width (GDK_PAINTABLE (svg));
+      height = gdk_paintable_get_intrinsic_height (GDK_PAINTABLE (svg));
+      if (width == 0 || height == 0)
         {
           g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
                        "Symbolic icon has no intrinsic size; please set one in its SVG");
-          g_object_unref (handle);
+          g_object_unref (svg);
           return NULL;
         }
-
-      width = (int) ceil (w);
-      height = (int) ceil (h);
     }
 
   if (!svg_has_symbolic_classes (bytes))
     {
-      texture = gdk_texture_new_from_rsvg (handle, width, height, error);
+      texture = svg_to_texture (svg, width, height, NULL, 0);
 
       if (texture)
         texture = keep_alpha (texture);
@@ -382,7 +274,7 @@ gdk_texture_new_from_bytes_symbolic (GBytes    *bytes,
       if (out_only_fg)
         *out_only_fg = TRUE;
 
-      g_object_unref (handle);
+      g_object_unref (svg);
 
       return texture;
     }
@@ -409,7 +301,7 @@ gdk_texture_new_from_bytes_symbolic (GBytes    *bytes,
        * channels, with the color of the fg being implicitly
        * the "rest", as all color fractions should add up to 1.
        */
-      loaded = load_symbolic_svg (handle, width, height,
+      loaded = load_symbolic_svg (svg, width, height,
                                   g_string,
                                   plane == 0 ? r_string : g_string,
                                   plane == 1 ? r_string : g_string,
@@ -438,7 +330,7 @@ gdk_texture_new_from_bytes_symbolic (GBytes    *bytes,
 
 out:
   g_bytes_unref (data_bytes);
-  g_object_unref (handle);
+  g_object_unref (svg);
 
   if (out_only_fg)
     *out_only_fg = only_fg;
@@ -559,18 +451,22 @@ gdk_texture_new_from_stream_at_scale (GInputStream  *stream,
                                       GCancellable  *cancellable,
                                       GError       **error)
 {
-  RsvgHandle *handle;
+  GBytes *bytes;
+  GtkSvg *svg;
   GdkTexture *texture;
 
   if (only_fg)
     *only_fg = FALSE;
 
-  handle = rsvg_handle_new_from_stream_sync (stream, NULL, RSVG_HANDLE_FLAGS_NONE, NULL, error);
-  if (!handle)
+  bytes = input_stream_get_bytes (stream, error);
+  if (!bytes)
     return NULL;
 
-  texture = gdk_texture_new_from_rsvg (handle, width, height, error);
-  g_object_unref (handle);
+  svg = gtk_svg_new_from_bytes (bytes);
+  texture = svg_to_texture (svg, width, height, NULL, 0);
+
+  g_object_unref (svg);
+  g_bytes_unref (bytes);
 
   return texture;
 }
@@ -705,14 +601,14 @@ gdk_paintable_new_from_bytes_scaled (GBytes *bytes,
     }
   else if (scale == 1)
     {
-      return GDK_PAINTABLE (gdk_texture_new_from_svg_bytes (bytes, scale, NULL));
+      return GDK_PAINTABLE (gdk_texture_new_from_svg_bytes (bytes, scale, NULL, 0, NULL));
     }
   else
     {
       GdkTexture *texture;
       GdkPaintable *paintable;
 
-      texture = gdk_texture_new_from_svg_bytes (bytes, scale, NULL);
+      texture = gdk_texture_new_from_svg_bytes (bytes, scale, NULL, 0, NULL);
       if (texture)
         {
           paintable = gtk_scaler_new (GDK_PAINTABLE (texture), scale);
@@ -789,6 +685,7 @@ typedef struct
   double width, height;
   GtkSnapshot *snapshot;
   gboolean only_fg;
+  gboolean has_strokes;
   gboolean has_clip;
   guint n_paths;
 } ParserData;
@@ -940,6 +837,7 @@ start_element_cb (GMarkupParseContext  *context,
   const char *stroke_dashoffset_attr = NULL;
   const char *opacity_attr = NULL;
   const char *class_attr = NULL;
+  const char *visibility_attr = NULL;
   GskPath *path = NULL;
   GskStroke *stroke = NULL;
   GskFillRule fill_rule;
@@ -951,6 +849,7 @@ start_element_cb (GMarkupParseContext  *context,
   char *end;
   gboolean do_fill = FALSE;
   gboolean do_stroke = FALSE;
+  gboolean visibility = TRUE;
 
   if (strcmp (element_name, "svg") == 0)
     {
@@ -1001,10 +900,14 @@ start_element_cb (GMarkupParseContext  *context,
            strcmp (element_name, "defs") == 0 ||
            strcmp (element_name, "style") == 0 ||
            g_str_has_prefix (element_name, "sodipodi:") ||
-           g_str_has_prefix (element_name, "inkscape:"))
-
+           g_str_has_prefix (element_name, "inkscape:") ||
+           strcmp (element_name, "metadata") == 0 ||
+           g_str_has_prefix (element_name, "rdf:") ||
+           g_str_has_prefix (element_name, "cc:") ||
+           g_str_has_prefix (element_name, "dc:"))
     {
-      /* Do nothing */
+      if (GTK_DEBUG_CHECK (ICONTHEME))
+        gdk_debug_message ("Ignoring element in symbolic icon: <%s>", element_name);
       return;
     }
   else if (strcmp (element_name, "circle") == 0)
@@ -1188,6 +1091,7 @@ start_element_cb (GMarkupParseContext  *context,
                                  error,
                                  "class", &class_attr,
                                  "opacity", &opacity_attr,
+                                 "visibility", &visibility_attr,
                                  "fill-rule", &fill_rule_attr,
                                  "fill-opacity", &fill_opacity_attr,
                                  "stroke-width", &stroke_width_attr,
@@ -1204,6 +1108,12 @@ start_element_cb (GMarkupParseContext  *context,
                                  "color", NULL,
                                  "overflow", NULL,
                                  "d", NULL,
+                                 "x", NULL,
+                                 "y", NULL,
+                                 "width", NULL,
+                                 "height", NULL,
+                                 "rx", NULL,
+                                 "ry", NULL,
                                  "cx", NULL,
                                  "cy", NULL,
                                  "r", NULL,
@@ -1326,6 +1236,13 @@ start_element_cb (GMarkupParseContext  *context,
         }
     }
 
+  visibility = TRUE;
+  if (visibility_attr)
+    {
+      if (strcmp (visibility_attr, "hidden") == 0)
+        visibility = FALSE;
+    }
+
   if (fill_rule_attr && strcmp (fill_rule_attr, "evenodd") == 0)
     fill_rule = GSK_FILL_RULE_EVEN_ODD;
   else
@@ -1431,23 +1348,27 @@ start_element_cb (GMarkupParseContext  *context,
       gsk_stroke_set_dash_offset (stroke, offset);
     }
 
-  if (opacity != 1)
-    gtk_snapshot_push_opacity (data->snapshot, opacity);
-
-  if (do_fill)
+  if (visibility)
     {
-      data->n_paths++;
-      gtk_snapshot_append_fill (data->snapshot, path, fill_rule, &fill_color);
-    }
+      if (opacity != 1)
+        gtk_snapshot_push_opacity (data->snapshot, opacity);
 
-  if (do_stroke)
-    {
-      data->n_paths++;
-      gtk_snapshot_append_stroke (data->snapshot, path, stroke, &stroke_color);
-    }
+      if (do_fill)
+        {
+          data->n_paths++;
+          gtk_snapshot_append_fill (data->snapshot, path, fill_rule, &fill_color);
+        }
 
-  if (opacity != 1)
-    gtk_snapshot_pop (data->snapshot);
+      if (do_stroke)
+        {
+          data->n_paths++;
+          data->has_strokes = TRUE;
+          gtk_snapshot_append_stroke (data->snapshot, path, stroke, &stroke_color);
+        }
+
+      if (opacity != 1)
+        gtk_snapshot_pop (data->snapshot);
+    }
 
 cleanup:
   g_clear_pointer (&path, gsk_path_unref);
@@ -1476,6 +1397,7 @@ static GskRenderNode *
 gsk_render_node_new_from_bytes_symbolic (GBytes    *bytes,
                                          gboolean  *only_fg,
                                          gboolean  *single_path,
+                                         gboolean  *has_strokes,
                                          double    *width,
                                          double    *height,
                                          GError   **error)
@@ -1494,6 +1416,7 @@ gsk_render_node_new_from_bytes_symbolic (GBytes    *bytes,
 
   data.width = data.height = 0;
   data.only_fg = TRUE;
+  data.has_strokes = FALSE;
   data.snapshot = gtk_snapshot_new ();
   data.has_clip = FALSE;
   data.n_paths = 0;
@@ -1524,6 +1447,9 @@ gsk_render_node_new_from_bytes_symbolic (GBytes    *bytes,
   if (single_path)
     *single_path = data.n_paths == 1;
 
+  if (has_strokes)
+    *has_strokes = data.has_strokes;
+
   *width = data.width;
   *height = data.height;
 
@@ -1534,6 +1460,7 @@ GskRenderNode *
 gsk_render_node_new_from_resource_symbolic (const char *path,
                                             gboolean   *only_fg,
                                             gboolean   *single_path,
+                                            gboolean   *has_strokes,
                                             double     *width,
                                             double     *height)
 {
@@ -1548,7 +1475,7 @@ gsk_render_node_new_from_resource_symbolic (const char *path,
   if (!bytes)
     return NULL;
 
-  node = gsk_render_node_new_from_bytes_symbolic (bytes, only_fg, single_path, width, height, &error);
+  node = gsk_render_node_new_from_bytes_symbolic (bytes, only_fg, single_path, has_strokes, width, height, &error);
   g_bytes_unref (bytes);
   if (error)
     {
@@ -1564,6 +1491,7 @@ GskRenderNode *
 gsk_render_node_new_from_filename_symbolic (const char *filename,
                                             gboolean   *only_fg,
                                             gboolean   *single_path,
+                                            gboolean   *has_strokes,
                                             double     *width,
                                             double     *height)
 {
@@ -1580,7 +1508,7 @@ gsk_render_node_new_from_filename_symbolic (const char *filename,
     return NULL;
 
   bytes = g_bytes_new_take (text, len);
-  node = gsk_render_node_new_from_bytes_symbolic (bytes, only_fg, single_path, width, height, &error);
+  node = gsk_render_node_new_from_bytes_symbolic (bytes, only_fg, single_path, has_strokes, width, height, &error);
   g_bytes_unref (bytes);
   if (error)
     {
@@ -1634,6 +1562,100 @@ apply_weight (const GskStroke *orig,
   return stroke;
 }
 
+static gboolean
+restroke_node (GskRenderNode *node,
+               float          weight,
+               GtkSnapshot   *snapshot)
+{
+  switch ((int) gsk_render_node_get_node_type (node))
+    {
+    case GSK_CONTAINER_NODE:
+      for (guint i = 0; i < gsk_container_node_get_n_children (node); i++)
+        if (!restroke_node (gsk_container_node_get_child (node, i), weight, snapshot))
+          return FALSE;
+      return TRUE;
+
+    case GSK_TRANSFORM_NODE:
+      {
+        gboolean ret;
+
+        gtk_snapshot_save (snapshot);
+        gtk_snapshot_transform (snapshot, gsk_transform_node_get_transform (node));
+        ret = restroke_node (gsk_transform_node_get_child (node), weight, snapshot);
+        gtk_snapshot_restore (snapshot);
+
+        return ret;
+      }
+
+    case GSK_CLIP_NODE:
+      {
+        gboolean ret;
+
+        gtk_snapshot_push_clip (snapshot, gsk_clip_node_get_clip (node));
+        ret = restroke_node (gsk_clip_node_get_child (node), weight, snapshot);
+        gtk_snapshot_pop (snapshot);
+
+        return ret;
+      }
+
+    case GSK_OPACITY_NODE:
+      {
+        gboolean ret;
+
+        gtk_snapshot_push_opacity (snapshot, gsk_opacity_node_get_opacity (node));
+        ret = restroke_node (gsk_opacity_node_get_child (node), weight, snapshot);
+        gtk_snapshot_pop (snapshot);
+
+        return ret;
+      }
+
+    case GSK_FILL_NODE:
+      {
+        gboolean ret;
+
+        gtk_snapshot_push_fill (snapshot,
+                                gsk_fill_node_get_path (node),
+                                gsk_fill_node_get_fill_rule (node));
+        ret = restroke_node (gsk_fill_node_get_child (node), weight, snapshot);
+        gtk_snapshot_pop (snapshot);
+
+        return ret;
+      }
+      break;
+
+    case GSK_STROKE_NODE:
+      {
+        gboolean ret;
+        GskStroke *stroke;
+
+        stroke = apply_weight (gsk_stroke_node_get_stroke (node), weight);
+        gtk_snapshot_push_stroke (snapshot,
+                                  gsk_stroke_node_get_path (node),
+                                  stroke);
+        gsk_stroke_free (stroke);
+
+        ret = restroke_node (gsk_stroke_node_get_child (node), weight, snapshot);
+        gtk_snapshot_pop (snapshot);
+
+        return ret;
+      }
+
+    case GSK_COLOR_NODE:
+      {
+        graphene_rect_t bounds;
+        GdkRGBA color;
+
+        gsk_render_node_get_bounds (node, &bounds);
+        color = *gsk_color_node_get_color (node);
+        gtk_snapshot_append_color (snapshot, &color, &bounds);
+      }
+      return TRUE;
+
+    default:
+      return FALSE;
+    }
+}
+
 static gboolean
 recolor_node (GskRenderNode *node,
               const GdkRGBA *colors,
@@ -1773,6 +1795,30 @@ gsk_render_node_recolor (GskRenderNode  *node,
   return ret;
 }
 
+gboolean
+gsk_render_node_restroke (GskRenderNode  *node,
+                          float           weight,
+                          GskRenderNode **restroked)
+{
+  GtkSnapshot *snapshot;
+  gboolean ret;
+
+  if (gsk_render_node_get_node_type (node) == GSK_TEXTURE_NODE)
+    {
+      *restroked = NULL;
+      return FALSE;
+    }
+
+  snapshot = gtk_snapshot_new ();
+  ret = restroke_node (node, weight, snapshot);
+  *restroked = gtk_snapshot_free_to_node (snapshot);
+
+  if (!ret)
+    g_clear_pointer (restroked, gsk_render_node_unref);
+
+  return ret;
+}
+
 /* }}} */
 
 /* vim:set foldmethod=marker: */
